#include <ibsys.h>


/*
 * There is a Problem with Linux Semaphores: 
 *      - enabling Interrupts
 *      - call to down()
 *
 * could produce spurious segmentation faults, because up() is called
 * within this interrupt just before down() has been called
 * then wait will be called with empty queue.
 *
 * I worked over this code again to protect from such conditions
 * hope it works. :-)
 *
 *
 *
 */
#ifdef HP82335
extern int ccrbits;
#endif

extern uint8 ibirq;

static int IRQ_mask;

#if USEINTS
/*
 * GPIB interrupt service routine -- fast and simple
 */
IBLCL void ibintr(int irq, struct pt_regs *registerp )
{
int s;

#if DEBUG
	if (dbgMask & DBG_INTR)
	        printk("GPIB INTERRUPT! semaphore id = %d\n", espsemid.count);
#endif
#ifdef NIAT
	GPIBout(imr3, 0);		                /* disable interrupts */
#endif
#ifdef NIPCII
	GPIBout(imr2, 0);
#endif
#ifdef NIPCIIa
/*
 * Seem to need both imr1 and imr2 to be  reset to avoid getting in loop.
 */
	GPIBout(imr2, 0);
	GPIBout(imr1, 0);
#if DEBUG
        if(dbgMask & DBG_INTR) printk("GPIB-IRQ nipcii\n");
#endif
        /* reset shared interrupt circuit */
	outb(0xff , ( 0x2f0 | ibirq ) );

#endif
#ifdef HP82335
	GPIBout(imr1, 0);
	GPIBout(imr0, 0);

	GPIBout(ccr, ccrbits & ~HR_INTEN);  /* disable interrupt */
        GPIBout(ccir[0x37f7], 0x0);        /* clear card interrupt IP bit */
        GPIBout(ccr, ccrbits);              /* re-enable interrupts if set */
#if DEBUG
        if(dbgMask & DBG_INTR) printk("GPIB-IRQ hp82335\n");
#endif
#endif


	    up(&espsemid); /* wake up sleeping process */
}


/*
 * Wait for GPIB or timer interrupt.  Semaphore will be posted by
 * either ibintr or ibtmintr.
 */



IBLCL osWaitForInt(imr3mask)
int imr3mask;
{
        struct wait_queue wait = {current, NULL}; /*@*/

	DBGin("osWaitForInt");
	if (espsemid.count <= 0) {
	/*
	 *	If semaphore not already available, enable
	 *	requested interrupts and wait until it is...
	 */

	  espsemid.count = 1;
          down(&espsemid); /* spurious interrups calling up() while irq's are enabled ? */

          /* now it's time to enable board interrupts */
#ifdef NIAT    
	        DBGprint(DBG_DATA, ("imr3mask=0x%x  ", imr3mask));		
                GPIBout(imr3, imr3mask);
#endif
#ifdef NIPCII
                DBGprint(DBG_DATA, ("imr3mask=0x%x  ", imr3mask));		
		GPIBout(imr2, imr3mask); /* sorry */	        
#endif
#ifdef HP82335
#if 0
	        GPIBout(ccr, ccrbits & ~HR_INTEN);/* disable interrupt */
                GPIBout(ccir[0x37f7], 0x0);      /* clear card interrupt IP bit */
                GPIBout(ccr, ccrbits);            /* re-enable interrupts if set */
                DBGprint(DBG_DATA,("===>csr=0x%x",GPIBin(csr)));
#endif
                DBGprint(DBG_DATA, ("imr3mask=0x%x  ", imr3mask));		
		GPIBout(imr1, imr3mask); /* sorry */	        
#endif
		/* now push process to sleep */
		down(&espsemid);
		up(&espsemid); /* wakeup process */
	}
        else	{ 
	  espsemid.count = 0;
          espsemid.wait  = NULL;
	}
	DBGout();
}
#endif


